<?php
namespace EEPROM;

use Index\Autoloader;
use Index\Environment;
use Index\Data\ConnectionFailedException;
use Index\Data\DbTools;

define('PRM_STARTUP', microtime(true));
define('PRM_ROOT', __DIR__);
define('PRM_CLI', PHP_SAPI === 'cli');
define('PRM_DEBUG', is_file(PRM_ROOT . '/.debug'));
define('PRM_PUBLIC', PRM_ROOT . '/public');
define('PRM_SOURCE', PRM_ROOT . '/src');
define('PRM_LIBRARY', PRM_ROOT . '/lib');
define('PRM_UPLOADS', PRM_PUBLIC . '/data');
define('PRM_THUMBS', PRM_PUBLIC . '/thumb');

require_once PRM_LIBRARY . '/index/index.php';

Autoloader::addNamespace(__NAMESPACE__, PRM_SOURCE);
Environment::setDebug(PRM_DEBUG);

mb_internal_encoding('utf-8');
date_default_timezone_set('utc');

set_exception_handler(function(\Throwable $ex) {
    http_response_code(500);
    header('Content-Type: text/plain; charset=utf-8');
    if(PRM_DEBUG)
        echo (string)$ex;
    exit;
});

set_error_handler(function(int $errno, string $errstr, string $errfile, int $errline) {
    throw new \ErrorException($errstr, 0, $errno, $errfile, $errline);
    return true;
}, -1);

if(!is_dir(PRM_UPLOADS))
    mkdir(PRM_UPLOADS, 0775, true);
if(!is_dir(PRM_THUMBS))
    mkdir(PRM_THUMBS, 0775, true);

$configPath = PRM_ROOT . '/config.ini';

if(!is_file($configPath))
    die('EEPROM configuration is missing.');

Config::load($configPath);

if(!Config::has('Database', 'dsn'))
    die('EEPROM database is not configured.');

try {
    $db = DbTools::create(Config::get('Database', 'dsn'));
} catch(ConnectionFailedException $ex) {
    echo '<h3>Unable to connect to database</h3>';
    die($ex->getMessage());
}
