<?php
namespace EEPROM;

use Exception;
use JsonSerializable;
use Index\Data\IDbConnection;
use Index\Data\DbType;

class ApplicationNotFoundException extends Exception {}

final class Application implements JsonSerializable {
    public function __construct(
        private int $id = 0,
        private string $name = '',
        private int $created = 0,
        private int $sizeLimit = -1,
        private bool $allowSizeMultiplier = false,
        private int $expiry = -1,
    ) {}

    public function getId(): int {
        return $this->id;
    }

    public function getName(): string {
        return $this->name;
    }

    public function getCreated(): int {
        return $this->created;
    }

    public function getSizeLimit(): int {
        return $this->sizeLimit;
    }

    public function allowSizeMultiplier(): bool {
        return $this->allowSizeMultiplier;
    }

    public function getExpiry(): int {
        return $this->expiry;
    }

    public function jsonSerialize(): mixed {
        return [
            'id' => $this->id,
            'name' => $this->name,
            'size_limit' => $this->sizeLimit,
            'size_multi' => $this->allowSizeMultiplier,
            'expiry' => $this->expiry,
            'created' => date('c', $this->created),
        ];
    }

    public static function byId(IDbConnection $conn, int $appId): self {
        if($appId < 1)
            throw new ApplicationNotFoundException;

        $get = $conn->prepare(
            'SELECT `app_id`, `app_name`, `app_size_limit`, `app_expiry`, `app_allow_size_multiplier`,'
            . ' UNIX_TIMESTAMP(`app_created`) AS `app_created` FROM `prm_applications` WHERE `app_id` = ?'
        );
        $get->addParameter(1, $appId, DbType::INTEGER);
        $get->execute();
        $result = $get->getResult();

        if(!$result->next())
            throw new ApplicationNotFoundException;

        return new static(
            $result->getInteger(0),
            $result->getString(1),
            $result->getInteger(5),
            $result->getInteger(2),
            $result->getInteger(4) !== 0,
            $result->getInteger(3),
        );
    }
}
